# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import unicodedata
import six

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String


class AgilentBase(Splitted):

    SEPARATOR = ","


class AgilentHeader(AgilentBase):
    FIELDS = (String("header"),)


class AgilentOrder(AgilentBase):
    FIELDS = (
        String("well"),
        String("well_type"),
        String("dye"),
        String("target"),
        String("replicate"),
        String("sample_name"),
        String("well_name"),
    )


class AgilentResult(object):
    FIELDS = [
        String("well"),
        String("well_type"),
        String("sample_name"),
        String("well_name"),
        String("dye"),
        String("target"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            else:
                setattr(self, field.name, line[i])


class AgilentMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.HEADER: AgilentHeader,
        MessageV2.ORDER: AgilentOrder,
        MessageV2.RESULT: AgilentResult,
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf_data = unicodedata.normalize(
            "NFKD", six.text_type(data.replace("\u2206", "D"))
        ).encode("ascii", "ignore")
        if six.PY3:
            buf_data = buf_data.decode("ascii")
        buf = six.StringIO(buf_data)
        reader = csv.reader(buf, delimiter=str(","))
        for line in reader:
            if not len(line):
                continue
            if len(line) > 10:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {"code": "agilent", "label": "AGILENT ARIA"}

    def create(self):
        return AgilentMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = AgilentMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = self.create()
        samples = worklist.get_plan_de_plaque_worklist()
        if len(samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        elif len(samples) == 1:
            sample = samples[0]
            msg.add(type_message=MessageV2.HEADER, header="[Plate Setup]")
            msg.add(
                type_message=MessageV2.ORDER,
                well="Well",
                well_type="Well Type",
                dye="Dye",
                target="Target",
                replicate="Replicate",
                sample_name="Sample Name",
                well_name="Well Name",
            )
            for order in sample.orders:
                codes = order.code.split(";")
            well_types = {
                0: "Unknow",
                2: "NTC",
                3: "Calibrator",
                4: "Standard",
                5: "No RT",
                6: "NAC",
                7: "Buffer",
                8: "Homo Allele A",
                9: "Homo Allele B",
                10: "Hetero",
            }
            for well in sample.wells:
                for code in codes:
                    msg.add(
                        type_message=MessageV2.ORDER,
                        well="{}{}".format(well.ordinate, well.abscissa),
                        well_type=well_types.get(well.well_type),
                        dye=code,
                        target=code,
                        replicate=well.abscissa,
                        well_name=well.barcode,
                    )

        return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.well.strip() == "Well" and m.well_type.strip() == "Well Type":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(
                    sample_id=m.sample_name.strip(), rack=m.well, type="patient"
                )
                for index, result in enumerate(m.results):
                    if result == "N/A":
                        continue
                    if RESULTS_CODES[index] and result:
                        sample.results.add(
                            code="{}{}".format(m.dye, RESULTS_CODES[index].strip()),
                            value=result.strip(),
                        )
        return worklist
