# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.fixed import FixedLengthFormat, Multiple, String
from driverng.drivers.au.base import AuOrder, AuQuery, AuRerunOrder, AuBaseDriver

logger = logging.getLogger("driverng.dxc700")


class DxC700Result(FixedLengthFormat):
    TEST = [
        String("test_number", 3),
        String("reagent", 32),
        String("analysis_data", 9),
        String("data_flags", 8),
        String("electrode", 20),
    ]
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + AuOrder.PATIENT
        + [Multiple("tests", TEST)]
    )


class DxC700RerunResult(FixedLengthFormat):
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + [Multiple("tests", DxC700Result.TEST)]
    )


class DxC700QC(FixedLengthFormat):
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + [Multiple("tests", DxC700Result.TEST)]
    )


class DxC700Driver(AuBaseDriver):
    SPEC = {"code": "dxc700", "label": "DxC 700 AU"}

    @staticmethod
    def get_klass(code):
        if code in ("Sh", "SH"):
            return AuRerunOrder
        elif code[0] == "S":
            return AuOrder
        elif code[0] == "R":
            return AuQuery
        elif code in ("DQ", "DR", "DA"):
            return DxC700QC
        elif code in ("Dh", "DH"):
            return DxC700RerunResult
        elif code[0] == "D":
            return DxC700Result
        else:
            raise NotImplementedError

    def decode_message(self, message):
        if isinstance(message, AuQuery):
            return self.get_query_worklist(message)

        worklist = Worklist(context="result")
        if isinstance(message, DxC700Result) or isinstance(message, DxC700RerunResult):
            sample = self.get_result_worlist(message, worklist)
        elif isinstance(message, DxC700QC):
            sample = self.get_qc_worklist(message, worklist)
        else:
            raise NotImplementedError

        for test in message.tests:
            flags = []
            if test.data_flags:
                for flag in test.data_flags:
                    if flag.strip():
                        flags.append(flag)
            lots = []
            if test.test_number in ("097", "098", "099"):
                reagents = self.get_ise_reagents(test.reagent)
                electrodes = self.get_electrodes(test.electrode)
                for e in electrodes:
                    lots.append(
                        {"lot": electrodes.get(e), "name": e, "type": "reagent"}
                    )
            else:
                reagents = self.get_reagents(test.reagent)
            for reagent in reagents:
                lots.append({"lot": reagent, "type": "reagent"})
            sample.results.add(
                code=test.test_number, value=test.analysis_data, flags=flags, lots=lots
            )
            if test.test_number == "096":
                splitted = test.analysis_data.split()
                if len(splitted) == 3:
                    for code, value in zip(["L", "I", "H"], splitted):
                        sample.results.add(
                            code=code, value=value, flags=flags, lots=lots
                        )

        return worklist
