# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, String

logger = logging.getLogger("driverng.genie_lacar")


class GenieLacarResult(object):
    FIELDS = [
        String("well"),
        String("sample"),
        String("matrix"),
        String("genotype"),
        Multiple("peaks", (String("resu"),), separator="\t"),
    ]

    def parse(self, line):
        for index, field in enumerate(self.FIELDS):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            setattr(self, field.name, line[index])


class GenieLacarMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: GenieLacarResult}

    def add(self, type_message=MessageV2.RESULT):
        obj = self._MESSAGE_TYPE_MAP[type_message]()
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str("\t"))
        self.date_result = None
        self.operator = None
        self.expiriment = None
        for line in reader:
            if not len(line) or len(line) == 1:
                continue
            if line[0].startswith("Experiment:"):
                self.experiment = line[1].split(" ")[0]
            elif line[0].startswith("User:"):
                self.operator = line[1].strip()
            elif line[0].startswith("Time:"):
                try:
                    self.date_result = datetime.datetime.strptime(
                        line[1].strip(), "%d %B %Y %H:%M"
                    )
                except ValueError as e:
                    logger.info('La date "Time"  n\'est pas au bon format %s', e)
            if len(line) > 3:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "genie_lacar", "label": "Genie LACAR CSV"}

    def parse(self, data):
        obj = GenieLacarMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            if m.sample == "Sample":
                continue
            if m.matrix in ("Positive control", "Negative control"):
                sample = worklist.get_or_create(
                    sample_id=m.matrix,
                    control_lot=m.matrix,
                    rack=m.well,
                    type="control",
                )
            else:
                sample = worklist.get_or_create(
                    sample_id=m.sample, rack=m.well, type="patient"
                )
            # Genotype
            if m.genotype:
                sample.results.add(
                    code="{}GENOTYPE".format(message.experiment),
                    value=m.genotype,
                    date=message.date_result,
                    operator=message.operator,
                )
            # Results peaks
            for index, result in enumerate(m.peaks, 1):
                sample.results.add(
                    code="{}PEAKS{}".format(message.experiment, index),
                    value=result,
                    date=message.date_result,
                    operator=message.operator,
                )
        return worklist
