# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import String


class LirasResult(object):
    separator = "\t"
    FIELDS = [
        String("limskey"),
        String("sampleid"),
        String("comment"),
        String("revision"),
        String("touched"),
        String("status"),
        String("td_id"),
        String("wl_id"),
        String("wl_name"),
        String("pa_code"),
        String("pa_name"),
        String("pa_comment"),
        String("pa_ethnicity"),
        String("pa_placeofbirth"),
        String("pa_dateofbirth"),
        String("ph_code"),
        String("ph_name"),
        String("ph_comment"),
        String("genotype"),
        String("messages"),
    ]

    def __init__(self, separator):
        self.separator = separator
        super(LirasResult, self).__init__()

    def parse(self, line):
        for index, field in enumerate(self.FIELDS):
            setattr(self, field.name, line[index])


class LirasMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: LirasResult}

    def __init__(self, separator):
        self.separator = separator
        super(LirasMultiple, self).__init__()

    def add(self, type_message=MessageV2.RESULT):
        obj = self._MESSAGE_TYPE_MAP[type_message](self.separator)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(self.separator))
        for line in reader:
            if not len(line) or len(line) == 1:
                continue
            obj = self.add(type_message=MessageV2.RESULT)
            obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "liras", "label": "LIRAS CSV"}
    OPTIONS = [
        MessageV2.Option(
            "separator", label="Séparateur des colonnes dans le fichier", default="\t"
        )
    ]

    def __init__(self, **kwargs):
        self.separator = kwargs.pop("separator", "\t")
        super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        obj = LirasMultiple(self.separator)
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            if m.limskey == "TS_LIMSKEY" and m.sampleid == "TS_SAMPLEID":
                continue
            sample = worklist.get_or_create(sample_id=m.sampleid, type="patient")
            # Flag
            flag = []
            if m.messages:
                flag.append(m.messages)
                sample.results.add(code="FLAG", value=m.messages)
            # TS_LIMSKEY
            sample.results.add(code="TS_LIMSKEY", value=m.limskey)
            # Genotype
            genotypes = m.genotype.split(",")
            for index, result in enumerate(genotypes, 1):
                if "(" not in result:
                    sample.results.add(
                        code="TS_GENR{}".format(index),
                        value=result,
                        dilution=m.comment,
                        flags=flag,
                    )
                else:
                    sample.results.add(
                        code="TS_GENR{}".format(index),
                        value=result[1:-1],
                        dilution=m.comment,
                        flags=flag,
                    )
        return worklist
