# -*- coding: utf8 -*-
from __future__ import unicode_literals
import datetime
import logging

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String, Date
from driverng.fixed import String as StringFixed

logger = logging.getLogger("driverng.lumipulse_g600ii")


class StringStrip(String):
    def to_python(self, value):
        return value.strip()


class LumipulseBase(Splitted):
    SEPARATOR = "|"


class LumipulseOrder(LumipulseBase):
    FIELDS = (
        StringFixed("type", 1, default="W"),  # W
        StringFixed("instrument", 2, default="00"),
        StringFixed("carousel_no", 2, empty="0"),
        StringFixed("sample_id", 20, empty="0"),
        StringFixed("specimen", 1, default="1"),
        StringFixed("manual_dilution", 4, default="0001", empty="0"),
        Multiple(
            "tests",
            (StringFixed("test_code", 3, empty="0"), StringFixed("auto_dilution", 1)),
            separator=LumipulseBase.SEPARATOR,
        ),
    )


class LumipulseQuery(LumipulseBase):
    FIELDS = [
        StringStrip("type"),  # R
        StringStrip("mode"),
        StringStrip("instrument"),
        StringStrip("carousel_no"),
        StringStrip("sample_id"),
    ]

    def parse(self, line):
        vals = line.split(self.SEPARATOR)
        for idx, f in enumerate(self.FIELDS):
            setattr(self, f.name, vals[idx])


class LumipulseResult(LumipulseBase):
    FIELDS = [
        StringStrip("type"),
        StringStrip("instrument"),
        StringStrip("sample_category"),
        StringStrip("batch_no"),
        StringStrip("sequence_no"),
        StringStrip("carousel_no"),
        StringStrip("sample_id"),
        StringStrip("specimen"),
        StringStrip("dilution"),
        StringStrip("nb_order_analyte"),
        StringStrip("order_analyte_no"),
        StringStrip("nb_multiple_assay"),
        StringStrip("multiple_assay_no"),
        StringStrip("analyte_no"),
        StringStrip("count_multiple_assay"),
        StringStrip("concentration_multiple_assay"),
        StringStrip("average_count_multiple"),
        StringStrip("calculated_concentration"),
        StringStrip("units"),
        StringStrip("error_analyte"),
        StringStrip("error_sample"),
        StringStrip("auto_dilution"),
        StringStrip("reagent_lot"),
        StringStrip("subtrate_lot"),
        Date("assay_date", fmt_date="%Y%m%d"),
        StringStrip("assay_time"),
        StringStrip("calibration_date"),
        StringStrip("calibration_time"),
    ]


class LumipulseMultiple(list):
    EXTENDED_RESULT = 99
    _MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: LumipulseOrder,
        MessageV2.RESULT: LumipulseResult,
        MessageV2.QUERY: LumipulseQuery,
    }
    SEPARATOR = "|"

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.split("\n"):
            if not len(line):
                continue
            if line[-1] == self.SEPARATOR:
                line = line[:-1]
            champs = line.split(self.SEPARATOR)
            if champs[0] == "D":
                type_message = MessageV2.RESULT
            elif champs[0] == "R":
                type_message = MessageV2.QUERY
            elif champs[0] == "W":
                type_message = MessageV2.ORDER
            else:
                logger.warning("message non reconnu: %s", line)
                continue
            obj = self.add(type_message=type_message)
            obj.parse(line)

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {"code": "lumipulse_g600ii", "label": "Lumipulse G600II de Fujirebio"}

    MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: LumipulseOrder,
        MessageV2.RESULT: LumipulseResult,
        MessageV2.QUERY: LumipulseQuery,
    }

    def create(self):
        return LumipulseMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = LumipulseMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        message = self.create()
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        if len(samples) == 0:
            return None
        for sample, action in samples:
            order_message = message.add(
                MessageV2.ORDER, sample_id=sample.sample_id, specimen=sample.specimen
            )
            if context["query"]:
                order_message.instrument = context["query"][0].instrument
                order_message.carousel_no = context["query"][0].carousel_no
            len_orders = len(sample["orders"])
            for order in sample["orders"]:
                order_message.tests.add(
                    test_code=order["code"], auto_dilution=order.dilution or "1"
                )
            # Il faut 8 tests par demande ou compléter par des valeurs à 0 si absence
            while len_orders < 8:
                order_message.tests.add(test_code="000", auto_dilution="0")
                len_orders += 1
        return message

    def decode_message(self, message):
        if message and isinstance(message[0], LumipulseQuery):
            worklist = Worklist(context="query")
            for m in message:
                if m.mode == "S":
                    worklist.get_or_create(type="patient", sample_id=m.sample_id)
            return worklist

        worklist = Worklist(context="result")
        if message and isinstance(message[0], (LumipulseResult,)):
            # Format RESULT FILE
            for m in message:
                if m.sample_category not in ("N", "Q"):
                    logger.warning("sample category non reconnu %s!", m.sample_category)
                    continue
                date_resu = None
                if m.assay_date:
                    if m.assay_time:
                        try:
                            time_resu = datetime.time(
                                int(m.assay_time[:2]),
                                int(m.assay_time[2:4]),
                                int(m.assay_time[4:]),
                            )
                            date_resu = m.assay_date.combine(m.assay_date, time_resu)
                        except (ValueError, IndexError):
                            date_resu = m.assay_date
                sample = worklist.get_or_create(
                    sample_id=m.sample_id,
                    specimen=m.specimen,
                    type="patient" if m.sample_category == "N" else "control",
                )
                if m.sample_category == "Q":
                    sample.control_lot = m.sample_id
                sample.rack = "{}_{}".format(m.sequence_no, m.carousel_no)
                flags = []
                if m.error_analyte:
                    flags.append(m.error_analyte)
                if m.error_sample and m.error_sample not in flags:
                    flags.append(m.error_sample)
                for test, value in [
                    ("COUNT", m.count_multiple_assay),
                    ("CONCENTRATION", m.concentration_multiple_assay),
                    ("AVERAGE", m.average_count_multiple),
                    ("CALCULATED", m.calculated_concentration),
                ]:
                    if value:
                        code = "{}{}".format(m.analyte_no, test)
                        resu = sample.results.add(
                            code=code,
                            value=value,
                            flags=flags,
                            dilution=m.auto_dilution,
                            unit=m.units,
                            instrument="g600ii" + m.instrument,
                            date=date_resu,
                        )
                        if m.reagent_lot:
                            resu.lots = [
                                {
                                    "type": "reagent",
                                    "lot": m.reagent_lot + "_" + m.subtrate_lot,
                                }
                            ]
        return worklist
