# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)


class MaglumiSegmentH(HeaderSegment):
    instrument_info = HprimString(4, default="PSWD")
    sender_id = HprimString(5, default="Maglumi 1000")
    receiver_id = HprimString(10, default="Lis")
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="E1394-97")
    date = HprimDate(14)

    patients = HprimForeign("P", "patients")
    queries = HprimForeign("Q", "queries")
    end = HprimForeign("L", "header", one2one=True, required=True)


class MaglumiSegmentP(Segment):

    orders = HprimForeign("O", "orders")


class MaglumiSegmentO(Segment):
    specimen_id = HprimString((3, 1))
    test_code = HprimString((5, 4))
    priority = HprimString(6)

    results = HprimForeign("R", "results")


class MaglumiSegmentR(Segment):
    test_code = HprimString((3, 4))
    value = HprimString(4)
    reference_range = HprimString(5)
    abnormal_flag = HprimString(7)
    date_result = HprimDate(13)


class MaglumiSegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    test_id = HprimString(5)
    request_information = HprimString(13)


class MaglumiSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):

    SPEC = {"code": "maglumi", "label": "Maglumi ASTM"}
    SEGMENT_CLASS_MAP = {
        "H": MaglumiSegmentH,
        "P": MaglumiSegmentP,
        "O": MaglumiSegmentO,
        "R": MaglumiSegmentR,
        "Q": MaglumiSegmentQ,
        "L": MaglumiSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "maglumi_instrument_info",
            label="Instrument information (champ H.4)",
            default="PSWD",
        ),
        AstmMessageV2.Option(
            "maglumi_sender_id", label="Sender Id (champ H.5)", default="Maglumi 1000"
        ),
        AstmMessageV2.Option(
            "maglumi_receiver_id", label="Receiver Id (champ H.10)", default="Lis"
        ),
    ]

    def __init__(self, **kwargs):
        self.instrument_id = kwargs.pop("maglumi_instrument_info", "PSWD")
        self.sender_id = kwargs.pop("maglumi_sender_id", "Maglumi 1000")
        self.receiver_id = kwargs.pop("maglumi_receiver_id", "Lis")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = MaglumiSegmentH(type_segment="H", **kwargs)
        message.end = MaglumiSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(
            instrument_id=self.instrument_id,
            sender_id=self.sender_id,
            receiver_id=self.receiver_id,
            date=context["now"],
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            seg_p = message.patients.add()
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test_code=order["code"],
                        priority=priority,
                    )
        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id)
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.specimen_id[0] == "#" and seg_o.specimen_id[-1] == "#":
                        sample = worklist.get_or_create(
                            control_lot=seg_o.specimen_id[1:-1],
                            sample_id=seg_o.specimen_id[1:-1],
                            type="control",
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.specimen_id, type="patient"
                        )
                    for seg_r in seg_o.results:
                        flags = []
                        if seg_r.abnormal_flag:
                            flags.append(seg_r.abnormal_flag)
                        sample.results.add(
                            code=seg_r.test_code,
                            value=seg_r.value,
                            flags=flags,
                            date=seg_r.date_result,
                        )
        return worklist
