# -*- coding: utf8 -*-
from __future__ import unicode_literals
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimMultiple,
    HprimString,
    HprimList,
    Segment,
)


class XnVSegmentH(HeaderSegment):
    analyzer_name = HprimString((5, 1))
    sofware_version = HprimString((5, 2))
    analyzer_serial = HprimString((5, 3))
    ps_code = HprimString((5, 4))
    version = HprimString(13, default="E1394-97")

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class XnVSegmentP(Segment):
    patient_id = HprimString(5)
    first_name = HprimString((6, 1))
    last_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M: Male
    # F: Female
    # U: Unknown
    sex = HprimString(9)
    physician = HprimString(14)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class XnVSegmentQ(Segment):
    rack = HprimString((3, 1))
    position = HprimString((3, 2))
    sample_id = HprimString((3, 3))
    sample_no = HprimString((3, 4))
    begin_request_date = HprimDate(7)
    request_status = HprimString(13)


class XnVSegmentO(Segment):
    rack = HprimString((3, 1))
    position = HprimString((3, 2))
    sample_id = HprimString((3, 3))
    species_id = HprimString((3, 4))
    instrument_rack = HprimString((4, 1))
    instrument_position = HprimString((4, 2))
    instrument_sample_id = HprimString((4, 3))
    instrument_species_id = HprimString((4, 4))
    test_id = HprimList(5, basefield=HprimMultiple(0, parameter_id=HprimString(4)))
    request_date = HprimDate(7)
    # N: New
    # A: Rerun
    # Q: QC
    action_code = HprimString(12)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class XnVSegmentR(Segment):
    parameter_id = HprimString((3, 5))
    dilution = HprimString((3, 6))
    analysis_result_type = HprimString((3, 7))
    extended_order_result = HprimString((3, 9))
    value = HprimString(4)
    unit = HprimString(5)
    abnormal_flags = HprimList(7)
    result_status = HprimString(9)
    date_result = HprimDate(13)


class XnVSegmentC(Segment):
    comment_text_no = HprimString((4, 1))
    comment_text_name = HprimString((4, 2))


class XnVSegmentS(Segment):
    reagents = HprimString((4, 1))
    lot_no = HprimString((4, 2))
    expirate_date = HprimDate((4, 3))
    expirate_date_after_open = HprimDate((4, 4))
    registration_date = HprimDate((4, 5))
    ru_name = HprimDate((4, 6))
    rack = HprimString((10, 1))
    position = HprimString((10, 2))
    sample_id = HprimString((10, 3))
    sample_no = HprimString((10, 4))
    result_date = HprimDate(15)


class XnVSegmentL(Segment):
    termination_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "xn-v", "label": "XN-V Series Vétérinaire ASTM"}
    SEGMENT_CLASS_MAP = {
        "H": XnVSegmentH,
        "P": XnVSegmentP,
        "Q": XnVSegmentO,
        "O": XnVSegmentO,
        "R": XnVSegmentR,
        "C": XnVSegmentC,
        "S": XnVSegmentS,
        "L": XnVSegmentL,
    }

    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = XnVSegmentH(type_segment="H", **kwargs)
        message.end = XnVSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "rerun"],
            action={"del": worklist.IGNORE, "new": "N", "add": "A"},
        ):
            physician = [
                p for p in patient.physicians if p.type.lower() == "prescriber"
            ]
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                location=patient.location,
                physician=physician[0].code if physician else "",
            )
            for sample, action, rerun in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    action_code=action,
                    request_date=sample.request_date,
                    report_type="Q",
                )
                seg_o.species_id = patient.animal_breed_code
                for order in sample.orders:
                    t = seg_o.test_id.add()
                    t.parameter_id = order.code
                    if order.action == "add" and rerun:
                        seg_o.action_code = "A"
        if message.patients:
            return message
        elif context["query"] and context["query"].queries:
            for seg_q in context["query"].queries:
                seg_p = message.patients.add()
                seg_o = seg_p.orders.add(sample_id=seg_q.sample_id, report_type="Y")
            return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient", sample_id=seg_q.sample_id.strip()
                )
        else:
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.action_code == "Q":
                        sample = worklist.get_or_create(
                            sample_id=seg_o.sample_id.strip(),
                            control_lot=seg_o.sample_id.strip(),
                            type="control",
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.sample_id.strip(), type="patient"
                        )
                    if seg_o.rack and seg_o.position:
                        sample.rack = "{}-{}".format(seg_o.rack, seg_o.position)
                    for seg_r in seg_o.results:
                        flags = []
                        comments = []
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment_code_no:
                                comments.append(seg_c.comment_code_no)
                        if seg_r.abnormal_flags:
                            flags.extend(seg_r.abnormal_flags)
                        sample.results.add(
                            code=seg_r.parameter_id,
                            value=seg_r.value,
                            dilution=seg_r.dilution,
                            flags=flags,
                            unit=seg_r.unit,
                            date=seg_r.date_result,
                            comments=comments,
                            instrument=message.analyzer_name,
                        )
        return worklist
